package com.huimv.guowei.admin.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.databind.ser.Serializers;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.common.utils.ResultUtil;
import com.huimv.guowei.admin.entity.BaseBuilding;
import com.huimv.guowei.admin.entity.BaseDuckInfo;
import com.huimv.guowei.admin.entity.EnvDevice;
import com.huimv.guowei.admin.entity.EnvDeviceEquipment;
import com.huimv.guowei.admin.entity.vo.*;
import com.huimv.guowei.admin.mapper.*;
import com.huimv.guowei.admin.service.IBaseBuildingService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-02-13
 */
@Service
public class BaseBuildingServiceImpl extends ServiceImpl<BaseBuildingMapper, BaseBuilding> implements IBaseBuildingService {
    @Autowired
    private BaseBuildingMapper baseBuildingMapper;
    @Resource
    private EnvDeviceMapper envDeviceMapper;
    @Resource
    private EnvDataMapper envDataMapper;
    @Resource
    private EnergyElectricityMapper energyElectricityMapper;
    @Resource
    private EnergyWaterMapper energyWaterMapper;
    @Resource
    private EnvDeviceEquipmentMapper envDeviceEquipmentMapper;
    @Resource
    private BaseDuckInfoMapper baseDuckInfoMapper;
    @Resource
    private EnvRegularCallEggMapper envRegularCallEggMapper;

    @Override
    @Transactional
    public Result addBuilding(BaseBuilding baseBuilding) {
        if (this.count(new QueryWrapper<BaseBuilding>().eq("build_name",baseBuilding.getBuildName())) != 0) {
            return new Result(10001,"栋舍名已存在",false);
        }
        baseBuilding.setParentId(0);
        baseBuilding.setFType(1);
        this.save(baseBuilding);
        Integer cageNum = baseBuilding.getCageNum();
        List<BaseBuilding> cageList = new ArrayList<>();
        for (int i = 1; i <= cageNum; i++) {
            BaseBuilding building = new BaseBuilding();
            building.setParentId(baseBuilding.getId());
            building.setFType(2);
            building.setStageCode(baseBuilding.getStageCode());
            building.setBuildName(baseBuilding.getBuildName()+ i +"笼");
            building.setCageName(i +"笼");
            building.setFarmId(baseBuilding.getFarmId());
            cageList.add(building);
        }
        this.saveBatch(cageList);
        return Result.SUCCESS();

    }

    @Override
    @Transactional
    public Result updateBuilding(BaseBuilding baseBuilding) {
        Integer cageNum = baseBuildingMapper.selectById(baseBuilding).getCageNum();
        if (cageNum > baseBuilding.getCageNum()){
            return new Result(10001,"笼位数量不能小于原值",false);
        }
        baseBuildingMapper.updateById(baseBuilding);
        Integer id = baseBuilding.getId();
        List<BaseBuilding> cageIds = baseBuildingMapper.selectList(new QueryWrapper<BaseBuilding>().eq("parent_id", id));
        if (ObjectUtil.isNotEmpty(cageIds)){
            for (BaseBuilding building : cageIds) {
                String newBuildName =  baseBuilding.getBuildName() + building.getCageName();
                building.setStageCode(baseBuilding.getStageCode());
                building.setBuildName(newBuildName);
                baseBuildingMapper.updateById(building);
                List<BaseDuckInfo> baseDuckInfos = baseDuckInfoMapper.selectList(new QueryWrapper<BaseDuckInfo>().eq("unit_id", building.getId()));
                if (ObjectUtil.isNotEmpty(baseDuckInfos)){
                    for (BaseDuckInfo baseDuckInfo : baseDuckInfos) {
                        baseDuckInfo.setUnitName(newBuildName);
                        baseDuckInfoMapper.updateById(baseDuckInfo);
                    }
                }
            }
        }
        for (int i = cageNum+1; i <= baseBuilding.getCageNum(); i++) {
            BaseBuilding building = new BaseBuilding();
            building.setBuildName(baseBuilding.getBuildName()+i+"笼");
            building.setParentId(Integer.valueOf(baseBuilding.getId()));
            building.setFType(2);
            building.setStageCode(baseBuilding.getStageCode());
            building.setFarmId(baseBuilding.getFarmId());
            building.setCageName(i+"笼");
            if (baseBuildingMapper.insert(building) != 1){
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return new Result(10001,"操作失败",false);
            }
        }
        return ResultUtil.updateResult(1);
    }

    @Override
    @Transactional
    public Result deleteBuilding(Map<String, Integer> map) {
        Integer id = map.get("id");
        List<Integer> cageIds = baseBuildingMapper.getCageById(id);
        if (ObjectUtil.isNotEmpty(cageIds)){
            if (ObjectUtil.isNotEmpty(baseDuckInfoMapper.selectList(new QueryWrapper<BaseDuckInfo>().in("unit_id",cageIds)))) {
                return new Result(10001,"删除失败，该栋舍下属笼中存在鸭只",false);
            }
            if (ObjectUtil.isNotEmpty(envDeviceMapper.selectList(new QueryWrapper<EnvDevice>().in("unit_id",cageIds)))) {
                return new Result(10001,"删除失败，该栋舍下属笼中存在设备",false);
            }
        }
        if (ObjectUtil.isNotEmpty(envDeviceMapper.selectList(new QueryWrapper<EnvDevice>().eq("unit_id",id)))) {
            return new Result(10001,"删除失败，该栋舍存在设备",false);
        }
        if (this.removeById(id) && this.removeByIds(cageIds)){
            return new Result(10000, "删除成功", true);
        }else {
            return new Result(10001, "删除失败", true);
        }
    }

    @Override
    public Result listBuilding(Map<String, String> map) {
        String farmId = map.get("farmId");
        String stageCode = map.get("stageCode");
        String buildName = map.get("buildName");
        String pageNum = map.get("pageNum");
        String pageSize = map.get("pageSize");
        Page<BaseBuilding> page = new Page(Integer.valueOf(pageNum),Integer.valueOf(pageSize));
        Page<BaseBuilding> baseBuildingPage = this.page(page, new QueryWrapper<BaseBuilding>()
                .eq("farm_id", farmId)
                .eq("f_type",1)
                .eq(StringUtils.isNotBlank(stageCode), "stage_code", stageCode)
                .like(StringUtils.isNotBlank(buildName), "build_name", buildName)
                .orderByAsc("id"));
        return new Result(ResultCode.SUCCESS,baseBuildingPage);
    }

    @Override
    public Result listTreeBuilding(Map<String, String> map) {
        String farmId = map.get("farmId");
        List<BaseBuilding> baseBuildings = this.list(new QueryWrapper<BaseBuilding>()
                .eq("farm_id", farmId)
                .orderByAsc("id"));
        List<TreeBaseBuilding> treeBaseBuildings = parseBizBaseArea(baseBuildings);
        return new Result(ResultCode.SUCCESS,treeBaseBuildings);
    }

    @Override
    @Transactional(rollbackFor = RuntimeException.class)
    public Result addSubBuilding(Map<String, String> map) {
        String farmId = map.get("farmId");
        String parentId = map.get("parentId");
        String num = map.get("cageNum");
        String stageCode = map.get("stageCode");

        BaseBuilding baseBuilding = baseBuildingMapper.selectById(parentId);
        String buildName = baseBuilding.getBuildName();

        List<BaseBuilding> baseBuildingList = baseBuildingMapper.selectList(new QueryWrapper<BaseBuilding>().eq("parent_id", parentId));
        System.out.println(num);
        for (int i = 0; i < Integer.parseInt(num); i++) {
            BaseBuilding building = new BaseBuilding();
            int no = baseBuildingList.size()+i+1;
            building.setBuildName(buildName+no+"笼");
            building.setParentId(Integer.valueOf(parentId));
            building.setFType(2);
            building.setStageCode(Integer.valueOf(stageCode));
            building.setFarmId(Integer.parseInt(farmId));
            building.setCageName(no+"笼");
            if (baseBuildingMapper.insert(building) != 1){
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return new Result(10001,"操作失败",false);
            }
        }
        baseBuilding.setCageNum(baseBuilding.getCageNum()+Integer.parseInt(num));
        baseBuildingMapper.updateById(baseBuilding);
        return Result.SUCCESS();
    }

    @Override
    public Result getUnitEnvironment(Map<String, String> map) {
        String farmId = map.get("farmId");
        String unitId = map.get("unitId");
        EnvDataVo envDataVo = envDataMapper.getLastFlowByUnitId(farmId,unitId);
        Map resultMap = new HashMap();
        if (ObjectUtil.isNotEmpty(envDataVo)){
            resultMap.put("envTemp",envDataVo.getEnvTemp());
            resultMap.put("envHum",envDataVo.getEnvHum());
        }
        int fanStatus = 0,tpStatus = 0,sprayStatus = 0;
        resultMap.put("energyWater", energyWaterMapper.getUnitDayWater(farmId, unitId));
        resultMap.put("energyElectricity",energyElectricityMapper.getDayUnitElectricity(farmId, unitId));
        if (envDeviceEquipmentMapper.selectCount(new QueryWrapper<EnvDeviceEquipment>().eq("open_start",1).eq("equipment_type",1).eq("unit_id",unitId)) >= 1) {
            fanStatus = 1;
        }
        if (envDeviceEquipmentMapper.selectCount(new QueryWrapper<EnvDeviceEquipment>().eq("open_start",1).eq("equipment_type",2).eq("unit_id",unitId)) >= 1) {
            tpStatus = 1;
        }
        if (envDeviceEquipmentMapper.selectCount(new QueryWrapper<EnvDeviceEquipment>().eq("open_start",1).eq("equipment_type",3).eq("unit_id",unitId)) >= 1) {
            sprayStatus = 1;
        }
        resultMap.put("fanStatus",fanStatus);
        resultMap.put("tpStatus",tpStatus);
        resultMap.put("sprayStatus",sprayStatus);

        List<EnvDeviceEquipment> deviceList =envDeviceEquipmentMapper.selectList(new QueryWrapper<EnvDeviceEquipment>().eq("unit_id",unitId).orderByAsc("equipment_port"));
        resultMap.put("deviceList",deviceList);
        return new Result(ResultCode.SUCCESS,resultMap);
    }

    @Override
    public Result getScreenEnvironment(Map<String, String> map) {
        String farmId = map.get("farmId");
        List<EnvDataScreenVo> envDataScreenVos = envDataMapper.getLastFlowScreen(farmId);
        for (EnvDataScreenVo envDataScreenVo : envDataScreenVos) {
            if (envDeviceEquipmentMapper.selectCount(new QueryWrapper<EnvDeviceEquipment>()
                    .eq("unit_id",envDataScreenVo.getId()).eq("equipment_type",1).eq("open_start",1)) > 0) {
                envDataScreenVo.setFanStatus(1);
            }else {
                envDataScreenVo.setFanStatus(0);
            }
            if (envDeviceEquipmentMapper.selectCount(new QueryWrapper<EnvDeviceEquipment>()
                    .eq("unit_id",envDataScreenVo.getId()).eq("equipment_type",3).eq("open_start",1)) > 0) {
                envDataScreenVo.setSprayStatus(1);
            }else {
                envDataScreenVo.setSprayStatus(0);
            }
        }
        return new Result(ResultCode.SUCCESS,envDataScreenVos);
    }

    @Override
    public Result listScreenEnergy() {
         List<ScreenEnergyVo> screenEnergyVos = baseBuildingMapper.listScreenEnergy();
        return new Result(ResultCode.SUCCESS,screenEnergyVos);
    }

    @Override
    public Result listScreenFeed(Map<String, String> map) {
        String farmId = map.get("farmId");
        Map resultMap = new HashMap();
        List<ScreenFeedVo> screenEnergyVos = baseBuildingMapper.listScreenFeed();
        resultMap.put("list",screenEnergyVos);
        String totalEgg = envRegularCallEggMapper.getTotalEgg(farmId);
        resultMap.put("totalEgg",totalEgg);
        return new Result(ResultCode.SUCCESS,resultMap);
    }

    @Override
    public Result listScreenDuck(Map<String, String> map) {
        String farmId = map.get("farmId");
        Map resultMap = new HashMap();
        List<ScreenDuckVo> screenDuckVos = baseBuildingMapper.listScreenDuck(farmId);
        resultMap.put("list",screenDuckVos);
        Integer count = baseDuckInfoMapper.selectCount(new QueryWrapper<BaseDuckInfo>().eq("farm_id", farmId));
        resultMap.put("totalDuck",count);
        return new Result(ResultCode.SUCCESS,resultMap);
    }


//    @Override
//    public Result list(String farmCode, String buildName, String stageCode) {
//        QueryWrapper<BaseBuilding> queryWrapper = new QueryWrapper<>();
//        queryWrapper.like(StringUtils.isNotBlank(buildName),"build_name", buildName);
//        queryWrapper.like(StringUtils.isNotBlank(stageCode),"stage_code", stageCode);
//        queryWrapper.eq(StringUtils.isNotBlank(farmCode),"farm_id", farmCode);
//        queryWrapper.orderByAsc("sort");
//        //创建排序
//        List<BaseBuilding> baseBuildings = baseBuildingMapper.selectList(queryWrapper);
//        //将结果List改为树
//        List<TreeBasePigpen> treeBasePigpens = parseBizBaseArea(baseBuildings);
//        return new Result(ResultCode.SUCCESS,treeBasePigpens);
//    }

    /**
     * 查询结果 转换成树形结构
     * @param bizBaseAreas 原始数据
     * @return 树
     */
    private List<TreeBaseBuilding> parseBizBaseArea(List<BaseBuilding> bizBaseAreas){
        //构建需要展示的树形节点结构
        Map<String, TreeBaseBuilding> nodeMap = bizBaseAreas.stream().map(baseBuilding -> {
            TreeBaseBuilding baseVo = new TreeBaseBuilding();
            baseVo.setId(baseBuilding.getId()+"");
            baseVo.setFarmCode(baseBuilding.getFarmId());
            baseVo.setParentId(baseBuilding.getParentId());
            baseVo.setSort(baseBuilding.getSort());
            baseVo.setStageCode(baseBuilding.getStageCode());
            baseVo.setBuildName(baseBuilding.getBuildName());
            baseVo.setFType(baseBuilding.getFType());
            baseVo.setChildNode(new ArrayList<>());
            return baseVo;
        }).collect(Collectors.toMap(TreeBaseBuilding::getId, b -> b,(k1, k2)->k1));
        //创建数组存父亲节点
        ArrayList<TreeBaseBuilding> roots = new ArrayList<>();
        //构建树形
        nodeMap.values().forEach(item->{
            String parentCode = item.getParentId()+"";
            if (nodeMap.get(parentCode) == null){
                //父节点为空，说明当前节点就已经是父节点了 将该节点存起来
                roots.add(item);
            }else {
                //父节点 不为空，说明有父节点，拿到该节点的父节点的孩子节点（就是我自己),将该节点存起来
                nodeMap.get(parentCode).getChildNode().add(item);
            }
        });
        roots.forEach(parent -> {
            List<TreeBaseBuilding> childNodes = parent.getChildNode();
            childNodes.sort(Comparator.comparingInt(o -> Integer.parseInt(o.getId())));
        });
        return roots;
    }
}
