package com.huimv.guowei.admin.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.druid.util.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.EnergyWaterPressure;
import com.huimv.guowei.admin.entity.EnvDeviceChip;
import com.huimv.guowei.admin.entity.vo.WaterLinePressureVo;
import com.huimv.guowei.admin.entity.vo.WaterLineVo;
import com.huimv.guowei.admin.mapper.EnergyWaterMapper;
import com.huimv.guowei.admin.mapper.EnergyWaterPressureMapper;
import com.huimv.guowei.admin.mapper.EnvDeviceChipMapper;
import com.huimv.guowei.admin.service.IEnergyWaterPressureService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-21
 */
@Service
public class EnergyWaterPressureServiceImpl extends ServiceImpl<EnergyWaterPressureMapper, EnergyWaterPressure> implements IEnergyWaterPressureService {

    @Resource
    private EnergyWaterPressureMapper energyWaterPressureMapper;
    @Resource
    private EnvDeviceChipMapper envDeviceChipMapper;

    @Override
    public Result getWaterPressure(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        List<EnvDeviceChip> deviceChipList = envDeviceChipMapper.selectList(new QueryWrapper<EnvDeviceChip>().eq("unit_id",unitId).eq("chip_type", 5));
        List<String> chipCodeList = new ArrayList<>();
        for (EnvDeviceChip envDeviceChip : deviceChipList) {
            chipCodeList.add(envDeviceChip.getChipCode());
        }
        String chipCodes = String.join(",", chipCodeList);
        WaterLinePressureVo waterLinePressureVo = new WaterLinePressureVo();
        waterLinePressureVo.setValue1("0");
        waterLinePressureVo.setValue2("0");
        waterLinePressureVo.setValue3("0");
        waterLinePressureVo.setValue4("0");
        chipCodes = "("+chipCodes+")";
        if (ObjectUtil.isNotEmpty(deviceChipList)){
            waterLinePressureVo = energyWaterPressureMapper.getWaterPressure(farmId,chipCodeList,chipCodes);
        }
        return new Result(ResultCode.SUCCESS,waterLinePressureVo);
    }

    @Override
    public Result listWaterPressure(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        String type = paramsMap.get("type");
        List<EnvDeviceChip> deviceChipList = envDeviceChipMapper.selectList(new QueryWrapper<EnvDeviceChip>().eq("unit_id",unitId).eq("chip_type", 5));
        List<String> chipCodeList = new ArrayList<>();
        for (EnvDeviceChip envDeviceChip : deviceChipList) {
            chipCodeList.add(envDeviceChip.getChipCode());
        }
        String chipCodes = String.join(",", chipCodeList);
        List<WaterLinePressureVo> waterLinePressureVo = new ArrayList<>();
        chipCodes = "("+chipCodes+")";
        if (ObjectUtil.isEmpty(deviceChipList)){
            return new Result(ResultCode.SUCCESS,waterLinePressureVo);
        }
        if ("1".equals(type)){
            waterLinePressureVo = energyWaterPressureMapper.listDayWaterPressure(farmId,chipCodeList,chipCodes);
        }else {
            waterLinePressureVo = energyWaterPressureMapper.listMonthWaterPressure(farmId,chipCodeList,chipCodes);
        }

        return new Result(ResultCode.SUCCESS,waterLinePressureVo);
    }

    @Override
    public Result listWaterLinePressure(Map<String, String> paramsMap) {
        String type = paramsMap.get("type");
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        if (StringUtils.isEmpty(unitId)){
            unitId = "232";
        }
        String startDate = paramsMap.get("startDate");
        String endDate = paramsMap.get("endDate")+" 23:59:59";
        List<WaterLinePressureVo> waterLineList = new ArrayList<>();
        List<EnvDeviceChip> deviceChipList = envDeviceChipMapper.selectList(new QueryWrapper<EnvDeviceChip>().eq("unit_id",unitId).eq("chip_type", 5));
        if (ObjectUtil.isEmpty(deviceChipList)){
            return new Result(ResultCode.SUCCESS,waterLineList);
        }
        List<String> chipCodeList = new ArrayList<>();
        for (EnvDeviceChip envDeviceChip : deviceChipList) {
            chipCodeList.add(envDeviceChip.getChipCode());
        }
        String chipCodes = String.join(",", chipCodeList);

        chipCodes = "("+chipCodes+")";
        if ("1".equals(type)){
            waterLineList = energyWaterPressureMapper.listWaterLineDayDetail(farmId,chipCodeList,chipCodes);
        }
        if ("2".equals(type)){
            waterLineList = energyWaterPressureMapper.listWaterLineWeekDetail(farmId,chipCodeList,chipCodes);
        }
        if ("3".equals(type)){
            waterLineList = energyWaterPressureMapper.listWaterLineMonthDetail(farmId,chipCodeList,chipCodes);
        }
        if ("4".equals(type)){
            waterLineList = energyWaterPressureMapper.listWaterLineCustomDetail(farmId,chipCodeList,chipCodes,startDate,endDate);
        }
        return new Result(ResultCode.SUCCESS,waterLineList);
    }
}
